"""
Testing suite for aat_wrest
"""

import argparse
import logging
import sys
from unittest.mock import patch, MagicMock

import pendulum as pendulum
from aat_wrest.metadata_wrester import WrestWorkflowMetadata

logger = logging.getLogger("test_aat_wrest")
logger.setLevel(logging.INFO)
logger.addHandler(logging.StreamHandler(sys.stdout))


args = argparse.Namespace(stdcals="uid://evla/calibration/143a3ef9-21fb-46d8-8312-91de54b0cc49")
result = (
    '{"sdmId": "17B-197.sb34663512.eb34806505.58108.78427738426", "projectCode": "17B-197", '
    '"title": "Radio-Active CoWS: Extended FIRST Sources in MaDCoWS Clusters", '
    '"startTime": 58108.7842916667, "observer": "Anthony Gonzalez", "telescope": "EVLA",'
    ' "created_at": ' + str(pendulum.now().in_timezone("UTC").format("YYYY-MM-DDTHH:mm:ss")) + "}"
)


def mock_wrester(args: argparse.Namespace) -> WrestWorkflowMetadata:
    with patch("psycopg2.connect") as mock_connect:
        return WrestWorkflowMetadata(connection=mock_connect, spl=args.stdcals)


wrester = mock_wrester(args)


class TestAatWrest:
    def test_init(self):
        fileset_id = "17B-197.sb34663512.eb34806505.58108.78427738426"
        with patch("psycopg2.connect") as mock_connect:
            mock_connect.cursor.return_value.__enter__.return_value.fetchall.return_value = [("uid://evla/calibration/143a3ef9-21fb-46d8-8312-91de54b0cc49",)]
            wrester_no_spl = WrestWorkflowMetadata(connection=mock_connect, spl=None, fileset_id=fileset_id)
            assert wrester_no_spl.spl == "uid://evla/calibration/143a3ef9-21fb-46d8-8312-91de54b0cc49"

    @patch("json.dumps", MagicMock(return_value=result))
    def test_wrest_standard_cal_info(self):
        wrester.conn.cursor.return_value.fetchall.return_value = [
            "17B-197.sb34663512.eb34806505.58108.78427738426",
            "17B-197",
            "Radio-Active CoWS: Extended FIRST Sources in MaDCoWS Clusters",
            58108.7842916667,
            "Anthony Gonzalez",
            "EVLA",
            str(pendulum.now().in_timezone("UTC").format("YYYY-MM-DDTHH:mm:ss")),
        ]
        assert args.stdcals == "uid://evla/calibration/143a3ef9-21fb-46d8-8312-91de54b0cc49"
        value = wrester.wrest_standard_cal_info()
        assert (
            value
            == '{"sdmId": "17B-197.sb34663512.eb34806505.58108.78427738426", "projectCode": "17B-197", '
            '"title": "Radio-Active CoWS: Extended FIRST Sources in MaDCoWS Clusters", '
            '"startTime": 58108.7842916667, "observer": "Anthony Gonzalez", "telescope": "EVLA",'
            ' "created_at": '
            + str(pendulum.now().in_timezone("UTC").format("YYYY-MM-DDTHH:mm:ss"))
            + "}"
        )

    def test_wrest_spl_from_fileset_id(self):
        wrester.conn.cursor.return_value.__enter__.return_value.fetchall.return_value = [("uid://evla/calibration/143a3ef9-21fb-46d8-8312-91de54b0cc49",)]
        spl = wrester.wrest_spl_from_fileset_id("17B-197.sb34663512.eb34806505.58108.78427738426")

        assert spl == "uid://evla/calibration/143a3ef9-21fb-46d8-8312-91de54b0cc49"