import { Component, OnInit } from "@angular/core";
import { CapabilityLauncherService } from "./services/capability-launcher.service";
import { CapabilityRequest } from "./model/capability-request";
import { CapabilityExecution } from "./model/capability-execution";
import { JsonObject } from "@angular/compiler-cli/ngcc/src/packages/entry_point";
import { ActivatedRoute, Router } from "@angular/router";
import { CapabilityRequestService } from "./services/capability-request.service";

@Component({
  selector: "app-workspaces",
  templateUrl: "./workspaces.component.html",
  styleUrls: ["./workspaces.component.scss"],
})
export class WorkspacesComponent implements OnInit {
  public productLocator: string;
  public userEmail: string;
  public inputFileList: FileList;

  constructor(
    private capabilityLauncher: CapabilityLauncherService,
    private capabilityRequestService: CapabilityRequestService,
    private router: Router,
    private route: ActivatedRoute,
  ) {
    // this is the famous 394 MB 13B-014 execblock, a useful smallish test
    this.productLocator = "uid://evla/image/3d3db489-9331-4e61-aa80-002bc2989b1e";
    this.userEmail = null;
  }

  ngOnInit(): void {}

  /**
   * OnClick method that creates a capability request for the null capability and submits it
   */
  nullButtonOnClick(): void {
    this.launchCapability("null", { arguments: "-g" });
  }

  /**
   * OnClick method that creates a capability request a given capability and submits it with the standard parameters:
   * - Product locator
   * - User email
   * FIXME: Remove check for calibration once it is implemented
   */
  standardLaunchCapabilityOnClick(capabilityName: string): void {
    this.launchCapability(capabilityName, {
      product_locator: this.productLocator,
      user_email: this.userEmail,
    });
  }

  /**
   * method that sets the user input Science Product Locator for the download capability
   * @param spl the Science Product Locator to download
   */
  setProductLocator(spl: string): void {
    this.productLocator = spl;
  }

  /**
   * method to set the user email for notifications
   * @param email user specified email
   */
  setUserEmail(email: string): void {
    this.userEmail = email;
  }

  /**
   * Set files to be uploaded to variable
   * @param inputFileList  List of files from modal form
   */
  public setInputFileList(inputFileList: FileList) {
    this.inputFileList = inputFileList;
  }

  /**
   * Method that uses the capabilityLauncher service to launch a capability
   * @param capabilityName  Name of capability
   * @param parameters      Parameters of capability request
   */
  launchCapability(capabilityName: string, parameters: JsonObject): void {
    const createRequestObserver = {
      next: (requestResponse: CapabilityRequest) => {
        if (requestResponse.id) {
          // Capability request created; ID found
          // Redirect to request status page
          if (this.inputFileList !== undefined) {
            this.capabilityRequestService.addFilesToVersion(requestResponse.id, this.inputFileList);
          }
          this.redirectToRequestStatusPage(requestResponse.id);
        }
      },
      error: (error) => {
        console.log(error);
      },
    };

    // Create capability request
    this.capabilityLauncher
      .createRequest(capabilityName, parameters)
      .subscribe(createRequestObserver);
  }

  /**
   * Redirect to status page of capability request with given ID
   * @param requestId  Given request ID
   */
  redirectToRequestStatusPage(requestId: string) {
    const statusPageUrl = this.router.serializeUrl(
      this.router.createUrlTree([`request-status/${requestId}`], {
        relativeTo: this.route,
      }),
    );
    // this.router.navigate([`request-status/${requestId}`], { relativeTo: this.route });
    // Open URL in new tab
    window.open(statusPageUrl, "_blank");
  }
}
