from unittest.mock import patch

from workspaces.products.services.archive_service import ArchiveService


class TestArchiveService:
    def test_on_ingestion_complete(self):
        ingestion_message = {
            "user": "test_user",
            "hostname": "hostname01",
            "timestamp": {
                "date": {"year": 2021, "month": 6, "day": 17},
                "time": {"hour": 18, "minute": 24, "second": 52, "nano": 18355000},
            },
            "application": "unknown",
            "request": "snek asks nice-like",
            "version": 1.0,
            "message": "Ingestion complete",
            "logData": {
                "ingestion_type": "rawdata",
                "execblock_id": "29223944",
                "override_db_screen": "False",
                "fileset_id": "13B-014.sb29151475.eb29223944.56810.442529050924",
                "telescope": "EVLA",
                "schedblock_id": "29151475",
                "schedblock_type": "SCIENCE",
                "project_code": "13B-014",
                "file_count": "1",
                "profile": None,
            },
        }

        expected_spl = "uid://evla/execblock/8fbfb54b-d141-42fe-b079-609339a69cba"
        with patch(
            "workspaces.products.services.archive_service.WrestWorkflowMetadata.wrest_spl_from_fileset_id",
            return_value=expected_spl,
        ) as mock_wrest_spl:
            mock_wrest_spl.return_value = expected_spl
            with patch("workspaces.products.services.archive_service.Router"):
                with patch(
                    "workspaces.products.services.archive_service.ArchiveService._create_std_calibration_request"
                ) as mock_create_req:
                    (
                        spl,
                        execblock_id,
                        fileset_id,
                        project_code,
                    ) = ArchiveService().on_ingestion_complete(**ingestion_message)
                    assert mock_create_req.call_count == 1
            assert spl == expected_spl
            assert execblock_id == "29223944"
            assert fileset_id == "13B-014.sb29151475.eb29223944.56810.442529050924"
            assert project_code == "13B-014"

    def test_create_std_calibration_request(self):
        spl = "uid://evla/execblock/8fbfb54b-d141-42fe-b079-609339a69cba"
        execblock_id = "29223944"
        fileset_id = "13B-014.sb29151475.eb29223944.56810.442529050924"
        project_code = "13B-014"
        params = {
            "parameters": {
                "product_locator": spl,
                "user_email": "workspaces-analysts@listmgr.nrao.edu",
                "execblock_id": execblock_id,
                "fileset_id": fileset_id,
                "project_code": project_code,
            }
        }

        with patch("workspaces.products.services.archive_service.Router"):
            with patch(
                "workspaces.products.services.archive_service.requests.post",
            ) as mock_post:
                ArchiveService()._create_std_calibration_request(
                    spl, execblock_id, fileset_id, project_code
                )
                mock_post.assert_called_with(
                    "http://capability:3457/capability/std_calibration/request/create", json=params
                )
